#ifndef _ARM_H
#define _ARM_H_

#include "globals.h"

#define ARM_FIRMWARE_VERSION "1.0 " __DATE__

#define ARM_COM 0 /* UART na kterem se posilaji prikazy (CKDM only) */

/* odpoved ziskana prikazem U16 (lze zjistit napr. pomoci programu inmotion
 * utilities) */
#define ARM_SERIES_NUMBER 52 

/* bitove masky pro arm_get_flags */
#define ARM_BIT_ALIVE ((uint32)1 << 0)
#define ARM_BIT_REFERENCED ((uint32)1 << 1)
#define ARM_BIT_BUSY ((uint32)1 << 2)
#define ARM_BIT_LOCAL ((uint32)1 << 3)
#define ARM_BIT_RANGE ((uint32)1 << 10)
#define ARM_BIT_RANGE_POSITIVE ((uint32)1 << 11)
#define ARM_BIT_RANGE_NEGATIVE ((uint32)1 << 12)
#define ARM_BIT_BLOCKED ((uint32)1 << 13)

/* Parametry pohybu ramene - viz manual radice krokoveho motoru. Bacha,
 * povolene jsou jen nektere kombinace start/stop a zrychleni! viz tabulku u
 * prikazu acceleration Uzivatelskeho manualu radice M1486, strana 30 */
struct arm_movement_params {
	uint32 start_stop;
	uint32 acceleration;
	uint32 velocity;
	uint32 profile;
};

enum arm_state {    /***/      /*****/
	stopped,      /*    rameno stoji           */
	go_positive,  /* nekonecny pohyb dopredu  */
	go_negative,  /* nekonecny pohyb dozadu  */
	go_forward,   /* omezeny pohyb dopredu  */
	go_backward,  /* omezeny pohyb dozadu  */	
	referencing   /* prave se referencuje */
};

extern uint32 arm_flags;

/* Parametry pohybu pro plnou a pomalou rychlost */
extern const struct arm_movement_params arm_movement_full;
extern const struct arm_movement_params arm_movement_slow;

/* Nejprve prikazy, ktere jsou soucasti komunikacniho protokolu ckdm-pc */

/* Do ukazatele na char hodi ukazatel na identifikator ramene. */
enum cmd_err
arm_identify(const char **id);

/* Precte status ramene. Viz makra ARM_BIT_* */
enum cmd_err
arm_get_flags(uint32 *flags);

/* Pokud je local true, rameno jde do lokalniho rezimu, jinak se lok. ovladani 
zakaze. */
enum cmd_err
arm_set_local(uint32 local);

/* Presune rameno na urcitou pozici [milistupne] */
enum cmd_err
arm_goto(int32 where);

/* Zastavi pohyb ramene */
enum cmd_err
arm_kill(void);

/* Zjisti polohu ramene [milistupne] */
enum cmd_err
arm_get_pos(int32 *pos);

/* Nastavi nulovou polohu ramene (viz protokol) */
enum cmd_err
arm_set_zero(int32 zero);

/* Zjisti nulovou polohu ramene (viz protokol) */
enum cmd_err
arm_get_zero(int32 *zero);

/* Zreferencuje rameno */
enum cmd_err
arm_reference(void);

/***************************************************************************/

/* Nasledujici prikazy nejsou (zatim) soucasti komunikacniho protokolu, jsou
 * hlavne kvuli rucnimu ovladani z panelu. */

/* Inicializace ramena */
enum cmd_err
arm_init(void);

/* Presun o [dist] _kroku_ od aktualni polohy. Hlavne kvuli tipovani na panelu,
 * take je o neco rychlejsi nez klasicke arm_goto. Vzdalenost by mela byt co
 * nejmensi (idealne pouze +- 1). Neni soucasti komunikacniho protokolu. */
enum cmd_err
arm_moverel(int32 where);

/* Rozbehne rameno v kladnem smeru */
enum cmd_err
arm_go_positive(void);

/* Rozbehne rameno v zapornem smeru */
enum cmd_err
arm_go_negative(void);

/* Test, zda jde rameno ovladat lokalne (tj. z panelu) */
enum cmd_err
arm_is_local(uint32 *local);

/* Nastavi rychlost (zrychleni, start/stop, profile...) pohybu ramene. */
enum cmd_err
arm_set_movement_params(const struct arm_movement_params *params);

/* Zjisti, co rameno prave dela */
enum cmd_err
arm_get_state(enum arm_state *state);

/* Zjisti, zda byl poslan a nedokoncen prikaz Kill */
enum cmd_err
arm_get_kill_sent(bool *kill_sent);

/* Precte vnitrni reprezentaci nuly, pouze kvuli zapisu/cteni do CMOS */
int32
arm_get_real_zero(void);

/* Ulozi vnitrni reprezentaci nuly, pouze kvuli zapisu/cteni do CMOS */
void
arm_set_real_zero(int32 zero);

#endif /* _ARM_H_ */
