#include "globals.h"
#include "buzzer.h"

volatile byte buzcnt;

void
buzzer_init(void)
{

	IO_PPGCAB.word &= ~0x8080; /* PPG enable - vypneme to. */
	
	IO_PPGCAB.bit.PE00	= 0; /* PPG ch0 output pin enable */
	IO_PPGCAB.bit.PE10	= 1; /* PPG ch1 output pin enable */
	IO_PPGCAB.bit.PIE0	= 1; /* PPG ch0 interrupt enable */
    IO_PPGCAB.bit.PIE1	= 0; /* PPG ch1 interrupt enable */
	IO_PPGCAB.bit.MD	= 0; /* 0 -- independent mode, 3 -- 16bit mode */

	/* BACHA, chyba v manualu co se tyce 
	 * casovych zakladen PCM a PCS, viz corrections k manualu ! 
	 * mam podezreni ze spatne jsou predevsim ty corrections :-) */
	IO_PPGAB.bit.PCM	= 7; /* casova zakladna ch0 */
	IO_PPGAB.bit.PCS	= 7; /* casova zakladna ch1 */
	
		/* bacha, kdyz v 16tibitovem modu pisu do PRL, MUSIM vzdy
		 * zapsat nejprve do ch0 a pak do ch1. Musim provest vzdy OBA
		 * zapisy !! (strana 174 HM dole) */
	IO_PRL[10].word		= 0x0000;
	
#if		defined(XTAL80000)
	IO_PRL[10].word		= 0x9B9B;
#elif	defined(XTAL73728)
	IO_PRL[10].word		= 0x909B;
#else
#error Neznamy krystal (makro XTAL*)
#endif
	
	IO_ICR11.bit.IL		= 4;

	IO_PPGCAB.bit.PEN0	= 1; /* jedem */

	buzzer_volume(1);
}


__interrupt void
buzzer_int(void)
{
	IO_PPGCAB.bit.PUF0 = 0;
	IO_PPGCAB.bit.PUF1 = 0;
	
	if (buzcnt)
		if (!--buzcnt)
			IO_PPGCAB.bit.PEN1 = 0;
}


void
buzzer(byte duration, byte pitch)
{
	if (!duration) {
		IO_PPGCAB.bit.PEN1 = 0;
		return;
	}
	IO_PRL[11].word = pitch | (pitch<<8);
	buzcnt = duration;
	IO_PPGCAB.bit.PEN1 = 1;
}

void
buzzer_ok(void)
{
	buzzer(50, 10);
}

void
buzzer_error(void)
{
	buzzer(50, 20);
}

/* zatim funguje jenom jako min/max (0/>0) */
void
buzzer_volume(byte vol)
{
	IO_DDR1.byte |= 0xC0;
	if (vol) {
		IO_PDR1.bit.P16 = 1;
		IO_PDR1.bit.P17 = 1;
	} else {
		IO_PDR1.bit.P16 = 0;
		IO_PDR1.bit.P17 = 0;
	}
}

