/*
 * gets.h
 *
 * Cteni retezcu z klavesnice krabice (SMS-like).
 *
 */

#ifndef _GETS_H_
#define _GETS_H_

#include "globals.h"

/* Vraci gets_key_filter */
enum gets_key_types {
	gets_normal,
	gets_illegal,
	gets_enter,
	gets_escape,
	gets_esc_unget,
	gets_backspace,
	gets_delete,
	gets_left,
	gets_right,
	gets_toggle_insert
};

/* p_gets pouziva pro rozpoznani znaku funkci gets_key_filter, ktera je typu
 * gets_key_filter_func a vraci jednu z hodnot gets_key_types. Pokud nevyhovuje
 * defaultni, je mozno priradit do gets_key_filter jinou. */
typedef enum gets_key_types (* gets_key_filter_func)(byte key);

/* Na to aby zadavana hodnota byla po stisknuti Enteru uznana, je treba
 * aby funkce gets_entry_filter volana s parameterem aktualni hodnoty
 * a jeji delky vratila nulu, jinak zabzucime a donutime uzivatele
 * napsat jinou hodnotu nebo utect pomoci K_ESC
 * (vhodne napr pro zadavani datumu, cisla... */
typedef byte (* gets_entry_filter_func)(char *entry, byte length);

/* Defaultni filtr klaves */
extern enum gets_key_types default_gets_key_filter(byte key);
extern gets_key_filter_func gets_key_filter;

/* Defaultni filtr vstupu (vraci vzdy nulu) */
extern gets_entry_filter_func gets_entry_filter;
extern byte default_gets_entry_filter(char *entry, byte length);

/* 0 -- replace, 1 -- insert (defaultne insert) */
extern byte gets_insert_mode;

/* Na aktualni pozici necha uzivatele natukat vstup (zatim) skrz sms_getkey.
 * Text ktery byl v buf je editovan, editovaci rezim je insert (a zatim nijak 
 * jinak). Umoznuje pohyb pomoci gets_left a gets_right, mazani pomoci gets_del
 * a gets_backspace, unik beze zmeny pomoci gets_esc a potvrzeni pomoci 
 * gets_enter.
 * Na gets_esc_unget je reagovano tak, ze se akt. edit. hodnota prozene
 * gets_entry_filter a at uz to dopadne jakkoliv, je p_ungetkey()nuta
 * posledni stiskla klavesa a pak return;.
 * Klavesy gets_ignore jsou ignorovany.
 * Delka textu nesmi prekrocit len nevcetne ukoncovaciho \0, tedy buf 
 * musi mit delku nejmene len+1, tato je dale je omezena pravym okrajem displeje
 * Po ukonceni editace se kurzor nevraci, puvodni obsah obrazovky neni obnoven.
 * Vraci ukazatel na buf.
 */
char *p_gets(char *buf, byte len);


/* pred gets(buf, len) skoci na [x,y] */
char *p_mvgets(byte x, byte y, char *buf, byte len);

#endif /* _GETS_H_ */
