#ifndef _SNOOP_H_
#define _SNOOP_H_

/*
 * Changelog:
 * 2006-10-05: Preformatovani komentaru
 * 2006-10-05: Pridano gpib_reset()
 * 2006-10-18: Zruseno gpib_ren, drat REN je nyni podle normy neustale v T 
 * 2006-11-27: Pridano GPIB_NRFD, GPIB_NOT_ALIVE, GPIB_NDAC
 */

	/* Negativni logika gpib */
#define GPIB_TRUE	0
#define GPIB_FALSE	1

	/*	zadna vyjimka */
#define GPIB_NONE			0
	/* preruseni uzivatelem */
#define GPIB_INTR			(1 << 0)
	/* preteceni gpib_buf */
#define GPIB_CMDOFLOW		(1 << 1)
	/* timeout */
#define GPIB_TIMEOUT		(1 << 2)
	/* prijemce nezije (NRFD == F && NDAC == F) */
#define GPIB_NOT_ALIVE		(1 << 3)
	/* prijemce neni pripraven na data (NRFD == F) */
#define GPIB_NRFD 		(1 << 4)
	/* prijemce neprijal data (NDAC == F) */
#define GPIB_NDAC		(1 << 5)

/* Moje GPIB adresa */
#define GPIB_ADDR		10
	/* timeout [jiffies] */
	/* Vetsinou zkousim akci 3x, takze 3x GPIB_MAXTIME musi byt o neco mensi
	 * nez timeout na PC. */
#define GPIB_MAXTIME	(JIFRATE/2)

extern byte gpib_cntrl; 

/* Jak tam mam naskladany draty ? */
/* vsechny ridici signaly v negativni logice (aktivni v nule !) */

#define GPIB_SETDATAIN	(IO_DDR0.byte = 0)
#define GPIB_SETDATAOUT	(IO_DDR0.byte = 0xff)
/* pozor, data na dat. sbernici jsou negovana, tyto funkce ji neguji
 * do 'spravneho' stavu. */
#define GPIB_GETDATA	(~(IO_PDR0.byte))
#define GPIB_SETDATA(b)	(IO_PDR0.byte = ~b)

#define GPIB_CNTRL	IO_PDR3.byte
#define GPIB_DCNTRL	IO_DDR3.byte

/* Talk Enable, Ovlivnuje budic kontrolni sbernice, rika jestli ma byt pro
 * cteni(0) ci pro zapis(1). Je zvlast pro datovou a ridici sbernici*/

#define GPIB_TE_DATA	IO_PDR1.bit.P10
#define GPIB_DTE_DATA	IO_DDR1.bit.D10
#define GPIB_TE_CTRL	IO_PDR1.bit.P11
#define GPIB_DTE_CTRL	IO_DDR1.bit.D11

#define GPIB_TE		GPIB_TE_DATA = GPIB_TE_CTRL
#define GPIB_DTE	GPIB_DTE_DATA = GPIB_DTE_CTRL

/* DAV, NRFD a NDAC mam pripojeny dvakrat, jednou abych na ne mohl psat a
 * podruhe abych z nich mohl cist (je mozne ze to co tam pisu neni to co na
 * sbernici skutecne je). */

/* Kontrolni draty pro zapis */
	/* data valid */
#define GPIB_DODAV	IO_DDR6.bit.D64
#define GPIB_ODAV	IO_PDR6.bit.P64
	/* not ready for data */
#define GPIB_DONRFD	IO_DDR6.bit.D65
#define GPIB_ONRFD	IO_PDR6.bit.P65
	/* not data accepted */
#define GPIB_DONDAC	IO_DDR6.bit.D66
#define GPIB_ONDAC	IO_PDR6.bit.P66

/* Kontrolni draty pro cteni */
	/* data valid */
#define GPIB_DIDAV	IO_DDR3.bit.D33
#define GPIB_IDAV	IO_PDR3.bit.P33
	/* not ready for data */
#define GPIB_DINRFD	IO_DDR3.bit.D34
#define GPIB_INRFD	IO_PDR3.bit.P34
	/* not data accepted */
#define GPIB_DINDAC	IO_DDR3.bit.D35
#define GPIB_INDAC	IO_PDR3.bit.P35

/* Ostatni ridici draty */
	/* service request */
#define GPIB_DSRQ	IO_DDR3.bit.D30
#define GPIB_SRQ	IO_PDR3.bit.P30
	/* attention */
#define GPIB_DATN	IO_DDR3.bit.D31
#define GPIB_ATN	IO_PDR3.bit.P31
	/* end of i?? */
#define GPIB_DEOI	IO_DDR3.bit.D32
#define GPIB_EOI	IO_PDR3.bit.P32
	/* interface clear */
#define GPIB_DIFC	IO_DDR3.bit.D36
#define GPIB_IFC	IO_PDR3.bit.P36
	/*	remote enable */
#define GPIB_DREN	IO_DDR3.bit.D37
#define GPIB_REN	IO_PDR3.bit.P37

/* Podmnozina MIM zprav */
#define MIM_GET		0x08	/* Group Execute Trigger */
#define MIM_UNL		0x3f	/* Unlisten */
#define MIM_UNT		0x5f	/* Untalk */
#define MIM_MLA(a)	(0x20 | (a))	/* My Listening Address */
#define MIM_MTA(a)	(0x40 | (a))	/* My Talking Address */
#define MIM_SPE		0x18		/* zacatek ser. hlaseni */
#define MIM_SPD		0x19		/* konec ser. hlaseni */
#define MIM_LLO		0x11		/* Local LockOut .. vypne panel vsech pristroju */
#define MIM_GTL		0x01		/* Go To Local .. povoli panel konkretniho pristroje */
#define MIM_DCL		0x14		/* Device Clear - clear zarizeni */

/* Provede pocatecni nastaveni gpib sbernice, melo by se volat pouze jednou na
 * zacatku programu. */
void
gpib_init(void);

/* Nastavi draty do definovaneho stavu, narozdil od gpib_init neprovadi napr.
 * nastaveni pinu jako GPIO atd... */
void
gpib_reset(void);

/* Vraci stav posledni gpib operace od vycisteni pomoci gpib_init().  Vraci
 * OR-nute GPIB_* vyjimky. */
uint16
gpib_get_status(void);

/* Posle na sbernici signal IFC (drat IFC na 100us do nuly) */
void
gpib_ifc(void);

/* Go To Local, zrejme pro vsechna zarizeni... */
void
gpib_gtl(char addr);

/* Group Execute Trigger pro zarizeni s adresou addr */
void
gpib_get(char addr);
	
/* Device Clear pro zarizeni s adresou addr */
void
gpib_dcl(char addr);

/* Zapise jednu multiline interface message (MIM), tedy napr.  mta, unl, get
 * atd.. Neni k zahozeni vyuzit MIM_* maker. Je vlastne stejne jako vyslani
 * bajtu, akorat assertne ATN.
 *
 * Mozne vyjimky:
 * EX_NONE
 * EX_INTR
 */
void
gpib_mim(byte msg);

/* 
 * Cte data az do EOI, vraci pocet prectenych dat. maxlen je max. pocet dat
 * ktera se mohou precist. Nedava na konec '\0'. Vraci pocet prectenych bajtu.
 *
 * Mozne vyjimky:
 * EX_NONE
 * EX_INTR
 * EX_CMDOFLOW
 */
size_t
gpib_read(byte addr, char *buf, size_t maxlen);

/* Zapise na GPIB string, nakonec posle \r a pote \n s EOI.
 * Vraci: vraci pocet zapsanych bajtu z str, NEvcetne \r\n
 *        Pokud nastala chyba, vraci zaporny chybovy kod
 *
 * Mozne vyjimky:
 * GPIB_NONE
 * GPIB_TIMEOUT
 * GPIB_INTR
 * GPIB_NOT_ALIVE
 * GPIB_NRFD
 * GPIB_NDAC
 */
size_t
gpib_wrstr(byte addr, const char *str);

/* Zkusi, zda nejake zarizeni drzi NRFD.
 *
 * NRFD Hold je jedna z moznosti, jak zjistit zda je zarizeni pripraveno
 * reagovat na GPIB prikazy, viz str. 65 IEEE 488. Pokud je zarizeni
 * naadresovano jako posluchac a zaroven drzi NRFD v T, je busy. 
 *
 * vraci true, pokud je zarizeni busy, false pokud neni.
 *
 * Mozne vyjimky:
 * GPIB_NONE
 * GPIB_TIMEOUT
 * GPIB_INTR
 */
bool
gpib_is_nrfd_held(byte addr);

#endif /* _SNOOP_H_ */
