#ifndef _LUXMETR_H_
#define _LUXMETR_H_

#include "globals.h"

#define LUX_BIT_ALIVE (1 << 0)
#define LUX_BIT_MEASURING (1 << 1)
#define LUX_BIT_MEASURED (1 << 2)
#define LUX_BIT_LOCAL (1 << 3)

#define LUX_ADDR	2	/* GPIB adresa luxxmetru */

/* Sirka okenka pro zjistovani kolisani osvetlenosti (vsechny hodnoty v okenku
 * musi projit testem in_tolerance */
#define LUX_WINDOW_WIDTH	3
/* Pokud hodnota na displeji prekroci LUX_OVERFLOW_VAL/1000, prepne se na vyssi
 * rozsah(pokud lze). */
#define LUX_OVERFLOW_VAL 1600
/* Pokud hodnota na displeji podkroci LUX_UNDERFLOW_VAL/1000, prepne se na
 * nizsi rozsah(pokud lze). */
#define LUX_UNDERFLOW_VAL 180

/* Vraci cmd_ok, pokud byl luxmetr nalezen, cmd_comm pokud ne. */
enum cmd_err
lux_init(void);

/* Odmeri hodnotu, kouka se jestli mereni prilis neplave. Mereni prochazi
 * postupne stavy Nemerim (LUX_BIT_MEASURING = 0, LUX_BIT_MEASURED = 0), Merim
 * (LUX_BIT_MEASURING = 1, LUX_BIT_MEASURED = 0), pote bud uspesne odmereno
 * (LUX_BIT_MEASURING = 0, LUX_BIT_MEASURED = 1) nebo neuspesne odmereno
 * (LUX_BIT_MEASURING = 0, LUX_BIT_MEASURED = 0).
 * Do stavu neuspesne odmereno se dostane pokud hodnota preteka nebo prilis
 * dlouho mereni kolisa (viz in_tolerance().
 *
 * Protoze mereni muze trvat dost dlouho, po kazdem odmeru vola callback().
 * Pokud ten vrati true, lux_measure se hned ukonci a nastavi stav neuspesne
 * odmereno, jinak pokracuje v odmerovani. Pokud callback=NULL, nevola se.
 */
enum cmd_err
lux_measure(bool (* callback)(void));

/* vraci odmerenou hodnotu */
enum cmd_err
lux_get_val(int32 *val);

enum cmd_err
lux_get_flags(uint32 *flags);

enum cmd_err
lux_set_local(bool local);

#endif /* _LUXMETR_H_ */
