#include <string.h>
#include "globals.h"
#include "panel.h"
#include "rtc.h"
#include "sleep.h"
#include "buzzer.h"

/*
 * Changelog
 * 2006-10-04: Nyni je mozne vypnout zhaseni displeje.
 * 2006-10-11: Includuju <string.h> misto "block.h"
 */
#include "mutex.h"

#ifdef DISPLAY_SAVING_TIMEOUT
volatile word	display_saving_timeout = DISPLAY_SAVING_TIMEOUT;
#endif 

volatile byte	KeybCnt;
volatile byte	KeybScan[4];
mutex_t			p_mutex;

const byte KeyMap[4][8]={
	{K_0,	K_7,	K_COMMA,K_BOX,	K_UP,	K_9,	K_8,	K_NONE},
	{K_ESC, K_F1,	K_NONE, K_F5,	K_F4,	K_F3,	K_F2,	K_NONE},
	{K_MENU,K_1,	K_NONE, K_RIGHT,K_ENTER,K_3,	K_2,	K_SHIFT},
	{K_LEFT,K_4,	K_DOWN, K_PLUS,	K_MINUS,K_6,	K_5,	K_NONE}
};

byte p_x; /* Souradnice kurzoru */
byte p_y;
char p_vram[P_ROWS][P_COLS]; /* offline videopamet */
char p_ungetkeybuf;
volatile byte key_pressed;
bool p_autorepeating;

/* Jeste nevim co dela :-), asi zapise 4 bity */
void 
wrins4(byte a)
{
	MUTEX_LOCK(p_mutex);
	IO_PDR2.byte=0; IO_PDR2.byte=0x10;
	IO_DDR2.byte=0xF1; IO_DDR2.byte=0xF3; IO_DDR2.byte=0xF7; IO_DDR2.byte=0xFF;
	a>>=4; IO_PDR2.byte=a|0x10; IO_PDR2.byte=a;
	IO_DDR2.byte=0xF7; IO_DDR2.byte=0xF3; IO_DDR2.byte=0xF3; IO_DDR2.byte=0xF0;
	MUTEX_UNLOCK(p_mutex);
}

/* Taky ne, mozna ceka az nebude busy flag ? */
int 
waitend4(void)
{
	int tck; byte a;
	tck=6000;
	MUTEX_LOCK(p_mutex);
	do {
		IO_DDR2.byte=0xF0;
		IO_PDR2.byte=0x20; IO_PDR2.byte=0x20; IO_PDR2.byte=0x30; IO_PDR2.byte=0x30;
		a=IO_PDR2.byte<<4&0xF0; a=IO_PDR2.byte<<4&0xF0; IO_PDR2.byte=0x20;
		IO_PDR2.byte=0x20; IO_PDR2.byte=0x30; IO_PDR2.byte=0x30;
		a=IO_PDR2.byte&0xF|a; a=IO_PDR2.byte&0xF|a;
		IO_PDR2.byte=0x20; IO_PDR2.byte=0x20; IO_PDR2.byte=0;
		IO_DDR2.byte=0xF0;
	} while(a&0x80 && --tck);
	MUTEX_UNLOCK(p_mutex);
	return !tck;
}

/* Zapise instrukci */
void 
wrinstr4(byte a)
{
	if(waitend4())
		return;
	MUTEX_LOCK(p_mutex);
	IO_PDR2.byte=0; IO_PDR2.byte=0; IO_PDR2.byte=0x10;
	IO_DDR2.byte=0xF1; IO_DDR2.byte=0xF3; IO_DDR2.byte=0xF7; IO_DDR2.byte=0xFF;
	IO_PDR2.byte=a>>4|0x10; IO_PDR2.byte=a>>4|0x10; IO_PDR2.byte=a>>4;
	IO_PDR2.byte=a>>4; IO_PDR2.byte=0x10; IO_PDR2.byte=0x10;
	IO_PDR2.byte=a&0xF|0x10; IO_PDR2.byte=a&0xF|0x10; IO_PDR2.byte=a&0xF; IO_PDR2.byte=a&0xF;
	IO_DDR2.byte=0xF7; IO_DDR2.byte=0xF3; IO_DDR2.byte=0xF1; IO_DDR2.byte=0xF0;
	MUTEX_UNLOCK(p_mutex);
}

/* Zapise data do DD RAM */
void 
wrdata4(byte a)
{
	if(waitend4())
		return;
	MUTEX_LOCK(p_mutex);
	IO_PDR2.byte=0x40; IO_PDR2.byte=0x40; IO_PDR2.byte=0x50;
	IO_DDR2.byte=0xF1; IO_DDR2.byte=0xF3; IO_DDR2.byte=0xF7; IO_DDR2.byte=0xFF;
	IO_PDR2.byte=a>>4|0x50; IO_PDR2.byte=a>>4|0x50; IO_PDR2.byte=a>>4|0x40;
	IO_PDR2.byte=a>>4|0x40; IO_PDR2.byte=0x50; IO_PDR2.byte=0x50;
	IO_PDR2.byte=a&0xF|0x50; IO_PDR2.byte=a&0xF|0x50; IO_PDR2.byte=a&0xF|0x40;
	IO_PDR2.byte=a&0xF|0x40; IO_PDR2.byte=0;
	IO_DDR2.byte=0xF7; IO_DDR2.byte=0xF3; IO_DDR2.byte=0xF1; IO_DDR2.byte=0xF0;
	MUTEX_UNLOCK(p_mutex);
}


void
p_init(void)
{
	long i;
	MUTEX_INIT(p_mutex);
	IO_DDR2.byte=0x60; IO_DDR2.byte=0x71; IO_DDR2.byte=0x77; IO_DDR2.byte=0xF7;
	IO_PDR2.byte=0;
	for(i=0;i<50000;i++); wrins4(0x30);
	for(i=0;i<10000;i++); wrins4(0x30);
	for(i=0;i<300;i++); wrins4(0x30);
	for(i=0;i<300;i++);
	wrinstr4(0x28); wrinstr4(0x8); wrinstr4(0x1); wrinstr4(0x6); wrinstr4(0xC);

	memset(p_vram, ' ', P_ROWS * P_COLS);
	p_move(0, 0);
	
	key_pressed = 0;
	p_ungetkeybuf = K_NONE;
	p_autorepeating = false;
}

void
p_move(byte x, byte y)
{
	wrinstr4(128 | DISPLAY_POS(x,y));
	p_x = x;
	p_y = y;
}

void
p_putc(char c)
{
	wrdata4(c);
	p_vram[p_y][p_x] = c;
	p_x++;
}

static byte
scan_keyrow(byte row)
{
	int c;
	for(c = 0; row & 128; c++, row <<= 1);
	return c;
}

byte 
p_getkey(void)
{
	byte k, c;
	static byte LastKey = K_NONE;

	if (p_ungetkeybuf != K_NONE) {
		k = p_ungetkeybuf;
		p_ungetkeybuf = K_NONE;
		goto ret;
	} 

		/* tohle prapodivne poradi slouzi k tomu, aby stisk klavesy
		 * shift byl detekovan az uplne naposled a fungoval mi autorepeat
		 * kombinace shift+klavesa. */
	if ((c = scan_keyrow(KeybScan[0])) < 8) {k = KeyMap[0][c]; goto found;}
	if ((c = scan_keyrow(KeybScan[1])) < 8) {k = KeyMap[1][c]; goto found;}
	if ((c = scan_keyrow(KeybScan[3])) < 8) {k = KeyMap[3][c]; goto found;}
	if ((c = scan_keyrow(KeybScan[2])) < 8) {k = KeyMap[2][c]; goto found;}

	LastKey = K_NONE;
	KeybCnt = 0;
	p_autorepeating = false;
	return K_NONE;
found:
	if (KeybCnt) 
		return K_NONE;
	
	if (LastKey == k) {
		KeybCnt = KEYB_RATE;
		p_autorepeating = true;
	} else {
		KeybCnt = KEYB_DELAY;
		p_autorepeating = false;
	}
	LastKey = k;
	
	buzzer(3, 50);
ret:
#ifdef DISPLAY_SAVING_TIMEOUT
	if (k == K_MENU) {
		if (IO_PDR7.bit.P75)
			display_saving_timeout = 1; /* support_int to zhasne */
		else {
			IO_PDR7.bit.P75 = 1;/* Podsviceni */
			display_saving_timeout = DISPLAY_SAVING_TIMEOUT;
		}
		return K_NONE;
	} else {
			IO_PDR7.bit.P75 = 1;/* Podsviceni */
			display_saving_timeout = DISPLAY_SAVING_TIMEOUT;
	}
#endif 
	return k;
};

/* vraci ktera klavesa je fyzicky stisknuta PRAVE TED (resp.
 * po poslednim oskenovani v support_int */
byte 
p_getpressedkey(void)
{
	byte k, r, c;

	for(r = 0; r < 4; r++) {
		k = KeybScan[r];
		for(c = 0; k & 128; c++, k <<= 1);
		if(c < 8)
			break;
	}
	if(r > 3) 
		return K_NONE;
	
	return KeyMap[r][c];
};

void
p_setcursor(byte type)
{
	wrinstr4(12|type);
}


void
p_done()
{
}

void 
p_clear(void)
{
	wrinstr4(1); /* Clear display */
	memset(p_vram[0], ' ', sizeof(p_vram));
	p_x = 0; p_y = 0;
}


void
p_mvputc(byte x, byte y, char c)
{
	p_move(x, y);
	p_putc(c);
}


void
p_atputc(byte x, byte y, char c)
{
	byte oldx = p_x;
	byte oldy = p_y;
	
	p_mvputc(x, y, c);
	p_move(oldx, oldy);
}


void
p_ungetkey(char c)
{
	p_ungetkeybuf = c;
}


void
p_flushrow(byte row)
{
	register byte oldx = p_x;
	register byte oldy = p_y;
	byte i;

	p_move(0, row);
	for (i = 0; i < P_COLS; i++)
		p_putc(p_vram[row][i]);
	p_move(oldx, oldy);
}

void
p_puts(const char *text)
{
	register char *tmp = p_vram[p_y] + p_x;
	char *start = tmp;
	register char c;

	c = *text++;
	while (c != '\0') {
		*tmp++ = c;
		c = *text++;
	}
	p_x += (tmp-start);
	p_flushrow(p_y);
}


void 
p_mvputs(byte x, byte y, const char *text)
{
	p_move(x, y);
	p_puts(text);
}

void 
p_atputs(byte x, byte y, const char *text)
{
	register byte oldx = p_x;
	register byte oldy = p_y;
	
	p_mvputs(x, y, text);
	p_move(oldx, oldy);
}

void
p_waitkey(byte key)
{
	if (key == K_ENTER_ESC) {
		for(;;) {
			switch(p_getkey()) {
				case K_ENTER: case K_ESC: return;
			}
			go_sleep();
		}
	} else if (key == K_ANY)
		while (p_getkey() == K_NONE) 
			go_sleep();
	else
		while (p_getkey() != key) go_sleep();
}

void
p_clearrow(byte row)
{
	memset(p_vram[row], ' ', P_COLS);
	p_flushrow(row);
}

void
p_putscreen(const char *t, bool clear)
{
	if (clear)
		p_clear();
	else
		p_move(0, 0);
	while (*t) {
		if (*t == '\n') {
			p_move(0, p_y + 1);
			if (t[1] != '\n' && t[1] != '\0')
				p_clearrow(p_y);
			t++;
		} else
			p_putc(*t++);
	}
}

void
p_putrow(byte row, const char *t)
{
	p_clearrow(row);
	p_mvputs(0, row, t);
}

void
p_programchar(byte chr, const byte *def)
{
	register byte i;
	
	wrinstr4(0x40 | (chr << 3)); /* Set CGRAM adress */
	for (i = 0; i < 8; i++)
		wrdata4(def[i]);
	p_move(p_x, p_y); /* Set DDRAM adress */
}

void
p_setled(char led, char state)
{
	switch(led) {
		case 0:
			IO_PDR7.bit.P76 = state & 1;
			IO_PDR7.bit.P77 = state >> 1;
			break;
		case 1:
			IO_PDR8.bit.P80 = state & 1;
			IO_PDR8.bit.P81 = state >> 1;
			break;
		case 2:
			IO_PDR8.bit.P82 = state & 1;
			IO_PDR8.bit.P83 = state >> 1;
			break;
		case 3:
			IO_PDR8.bit.P84 = state & 1;
			IO_PDR8.bit.P85 = state >> 1;
			break;
		case 4:
			IO_PDR8.bit.P86 = state & 1;
			IO_PDR8.bit.P87 = state >> 1;
			break;
	}
}

	
#if 0
void
p_low_power_prepare(void)
{
		/* cteci draty */
	IO_DDR5.bit.D51 = 0;
	IO_DDR5.bit.D52 = 0;
	IO_DDR5.bit.D53 = 0;
	IO_DDR5.bit.D54 = 0;
	IO_DDR9.bit.D92 = 0;
	IO_DDR9.bit.D93 = 0;
	IO_DDR9.bit.D94 = 0;
	IO_DDR9.bit.D95 = 0;
		/* nastavovaci draty */
	IO_PDR7.bit.P70 = 0;
	IO_PDR7.bit.P71 = 0;
	IO_PDR7.bit.P72 = 0;
	IO_PDR7.bit.P73 = 0;

		/* INT 0/1 */
	IO_ICR01.bit.ISE = 0; /* External interrupt, nikoliv DTP */
	IO_ICR01.bit.IL = 1;
		/* INT 2/3 */
	IO_ICR03.bit.ISE = 0;
	IO_ICR03.bit.IL = 1;
		/* INT 4/5 */
	IO_ICR04.bit.ISE = 0;
	IO_ICR04.bit.IL = 1;
		/* INT 6/7 */
	IO_ICR07.bit.ISE = 0;
	IO_ICR07.bit.IL = 1;
}
#endif
