#ifndef _PANEL_H_
#define _PANEL_H_

/*
 * Changelog
 * 2006-10-04: Nyni je mozne vypnout zhaseni displeje.
 */

#include "globals.h"
#include "num2str.h"


/** Je stisknuta klavesa Shift ? */
#define P_ISSHIFT (!(KeybScan[2] & 1))

/** Pouze pro cteni, je  true pokud prave probiha autorepeat 
 * (tj. klavesa byla drzena dele nez P_DELAY) */
extern bool p_autorepeating;

/** Timeout pro zhaseni displeje, v setinach sekundy. Pokud neni definovano,
 * dispej se nezhasi vubec. */
/* #define DISPLAY_SAVING_TIMEOUT	1000 */
/** jak dlouho se ma cekat po stisku klavesy nez rozjedu autorepeaty ? [s/100] */
#define KEYB_DELAY				70
/** a kdyz uz se rozjedu, jaka je pauza mezi jednotlivymi autorepeaty ? */
#define KEYB_RATE				20


/* Velikost displeje panelu */
#define P_ROWS 4
#define P_COLS 20

/* Konstanty definujici klavesy panelu */
#define K_NONE	0

#define K_ESC	27
#define K_ENTER	13

#define K_F1	1
#define K_F2	2
#define K_F3	3
#define K_F4	4
#define K_F5	5
#define K_SHIFT	6
#define K_MENU	7
#define K_BOX	8
#define K_UP	9
#define K_DOWN	10
#define K_LEFT	11
#define K_RIGHT	12

#define K_0		14
#define K_1		15
#define K_2		16
#define K_3		17
#define K_4		18
#define K_5		19
#define K_6		20
#define K_7		21
#define K_8		22
#define K_9		23

#define K_MINUS	24
#define K_PLUS	25
#define K_COMMA 26

#define K_ENTER_ESC	253		/* ESC/ENTER (pro p_waitkey) */
#define K_ANY		254		/* libovolna klavesa (napr. p_waitkey) */
#define K_NOTEXIST	255 	/* Neexistujici klavesa */


/* Typy kurzoru (C_BLINK a C_UNDERLINE mohou byt spojeny pomoci | )*/
#define C_NONE 0
#define C_BLOCK 1
#define C_UNDERLINE 2

/* Stavy LEDek (viz p_setled()) */
#define LED_NONE	0
#define LED_GREEN	1
#define LED_RED		2
#define LED_BOTH	3

/** Timeout pro zhaseni displeje, v setinach sekundy */
extern volatile word display_saving_timeout;
/** KeybCnt se obcas muze k necemu hodit -- po p_getkey pri stisknute klavese
 * ma 7 az 15 jiffies nenulovou hodnotu (viz p_getkey) */
extern volatile byte KeybCnt;
/* sem se ukladaj bajtovy hodnoty jednotlivych radek masky klavesnice */
extern volatile byte	KeybScan[4];
/* Kazde drzeni klavesy zpusobi ze ma nenulovou hodnotu, je zcela
 * na uzivateli aby si ji vynuloval az bude potrebovat, sama se nevynuluje */
extern volatile byte key_pressed; 


extern byte p_x; /* x-ova pozice kurzoru */
extern byte p_y; /* y-ova pozice kurzoru */
extern char p_vram[P_ROWS][P_COLS]; /* Videopamet */

/* Prevod souradnic na adresu v DD RAM displeje */
#define DISPLAY_POS(x,y) (((y)<<1&4 | (y)<<3&16 | (y)<<6&64) + (x))


/** p_init
 * Vyplni p_vram znakem ' ', inicializuje hardware panelu */
void
p_init(void);

/** p_move
 * Presune kurzor na pozici
 * @param x sloupec na displeji pocinaje 0
 * @param y radek na displeji pocinaje 0
 */
void
p_move(byte x, byte y);

/** p_putc
 * Vypise na akt. pozici znak, flushne jej, posune kurzor doprava 
 * @param c znak k vypsani na displej
 */
void
p_putc(char c);

/** p_mvputc
 * Vypise na pozici [x, y] znak, flushne jej, posune kurzor na [x+1,y]
 * @param x sloupec
 * @param y radek
 * @param c znak k vypsani
 */
void
p_mvputc(byte x, byte y, char c);

/** p_ungetkey
 * Pristi p_getkey vrati c, buffer je jednoznakovy.
 * @param c -- znak, ktery se ma vlozit do bufferu klavesnice
 */
void
p_ungetkey(char c);

/** p_atputc
 * Jako p_mvputc, ale kurzor zustava na stejne pozici jako pred volanim.
 * @param x sloupec
 * @param y radek
 * @param c znak k vypsani
 */
void
p_atputc(byte x, byte y, char c);

/** p_getkey
 * Precte klavesu (pokud neni zadna stisknuta, vraci K_NONE),
 * respektuje buffer pro p_ungetkey(1 byte), stara se o autorepeat
 * a o zhaseni/rozsveceni displeje (vcetne klavesy MENU)
 * @return kod klavesy nebo K_NONE
 */
byte
p_getkey(void);

/** p_getpressedkey
 * vraci ktera klavesa je fyzicky stisknuta PRAVE TED (resp.
 * po poslednim oskenovani v handleru casovace
 * @return kod klavesy nebo K_NONE
 */
byte 
p_getpressedkey(void);

/** p_flushrow
 * Synchronizuje radek row displeje s videopameti (p_vram -> displej) 
 * @param row radek k flushnuti z p_vram na displej
 */
void 
p_flushrow(byte row);

/** p_puts
 * Hodi na akt. pozici text a flushne akt. radek. Nekontroluje specialni
 * znaky ani preteceni radku ci cele pameti.
 * @param text text k vypsani
 */
void 
p_puts(const char *text);

/** p_mvputs
 * presune kurzor na [x,y] a provede p_puts(text)
 * @param x sloupec
 * @param y radek
 * @param text text k vypsani
 * */
void 
p_mvputs(byte x, byte y, const char *text);

/** p_atputc
 * jako p_mvputs ale kurzor se vraci na misto pred volanim fce.
 * @param x sloupec
 * @param y radek
 * @param text text k vypsani
 */
void 
p_atputs(byte x, byte y, const char *text);

/** p_setcursor
 * Nastavi tvar kurzoru (viz konst. C_*)
 * @param type typ kurzoru dle konstant C_*
 * */
void
p_setcursor(byte type);

/** p_clear
 * Vycisti displej, p_vram, kurzor na [0,0]
 */
void p_clear(void);

/** p_waitkey
 * Ceka na stisk klavesy, mezitim i spi (go_sleep())
 * @param key klavesa, na kterou se ceka, funguje i na K_ANY (lib. klavesa)
 */
void
p_waitkey(byte key);
	
/**
 * Vymezeruje radek row, nemeni akt. pozici kursoru
 * @param row radek k vymazani
 */
void
p_clearrow(byte row);
	
/**
 * Smaze radek row a prdne na nej text t
 * @param row urceny radek
 * @param t text
 * */
void
p_putrow(byte row, const char *t);

/** Naprogramuje uzivatelsky programovatelny znak.
 * @param chr znak k programovani c <0, 7> 
 * @param def 8 bajtu s definici odshora dolu, bit 0 je napravo, znak je 6x8
 */
void 
p_programchar(byte chr, const byte *def);

/** p_low_power_prepare
 * pripravi klavesnici na vysilani ext. preruseni 
 * (nutne na probuzeni ze stop modu) */
void
p_low_power_prepare(void);

/**
 * (clear == true => vycisti obrazovku a) 
 * prdne na ni text t, ve kterem za kazdym
 * znakem '\n' provede prechod na zacatek dalsi radky a tuto
 * vycisti, pokud na ni ma byt neco napsano (tj. dalsi znak neni
 * opet '\n' nebo '\0'.
 * @param *t text ve vyse popsanem formatu
 * @param clear pokud je true, vycisti se cela obrazovka, jinak
 * jen radky na ktere se bude neco psat.
 */
void
p_putscreen(const char *t, bool clear);

/**
 * Nastavi ledku led <0, 4> na jednu z LED_* konstant. 
 * @param led Ktera ledka se ma nastavit (0 je horni, 4 spodni)
 * @param state jedna z LED_* konstant 
 */
void
p_setled(char led, char state);

#endif /* _PANEL_H_ */
