#include "globals.h"
#include "printf.h"
#include <stdarg.h>
#include "panel.h"

void (*outc)(char c);

void
p_printf(const char *fmt, ...)
{
	va_list ap;
#ifndef __GNUC__
	UNUSED_ARG(__builtin_va_alist);
#endif 
	outc = p_putc;
	va_start(ap, fmt);
	vprintf(fmt, ap);
	va_end(ap);
}

#define TEMPSIZE 17

#define SIGN 		1 << 0
#define FLUSHLEFT	1 << 1
#define ISLONG		1 << 2
#define SHARP		1 << 3

#define digit_val(x) ((x) - '0')
#define is_digit(x) (!((x) < '0' || (x) > '9'))

void
vprintf(const char *f, va_list ap)
{
	static const char digit[] = "0123456789ABCDEF";
	char c;
	char temp[TEMPSIZE];
	int width;
	int prec;
	char *p;
	unsigned long num;
	unsigned base;
	int len;
	int pad;
	char flags;

	while ((c = *f++) != '\0') {
		if (c != '%') {
			outc(c);
			continue;
		}
		flags = 0;
		width = 0;
		prec = -1;
		for (;;) {
			if (*f == '-')
				flags |= FLUSHLEFT;
			else if (*f == '#')
				flags |= SHARP;
			else
				break;
			f++;
		}
		if (*f == '*') {
			width = va_arg(ap, int);
			f++;
		} else {
			while (is_digit(*f)) {
				width = 10 * width + digit_val(*f++);
			}
		}
		if (*f == '.') {
			if (*++f == '*') {
				prec = va_arg(ap, int);
				f++;
			} else {
				prec = 0;
				while (is_digit(*f)) {
					prec = 10 * prec + digit_val(*f++);
				}
			}
		}
		if (*f == 'l') {
			f++;
			flags |= ISLONG;
		}
		switch (*f) {
		case 'd':
			base = 10;
			flags |= SIGN;
			goto number;
		case 'u':
			base = 10;
			goto number;
		case 'o':
			base = 8;
			goto number;
		case 'p':
			outc('0');
			outc('x');
			/*FALLTHROUGH*/
		case 'x':
			/* we don't implement 'x'; treat like 'X' */
		case 'X':
			base = 16;
number:		  /* process a number */
			if (flags & ISLONG)
				num = va_arg(ap, unsigned long);
			else
				num = va_arg(ap, unsigned int);
			if (flags & SIGN) {
				if ((signed long)num < 0)
					num = (unsigned long) (-(signed long)num);
				else
					flags &= ~SIGN;
			}
			p = temp + TEMPSIZE - 1;
			*p = '\0';
			while (num) {
				*--p = digit[num % base];
				num /= base;
			}
			len = (temp + TEMPSIZE - 1) - p;
			if (prec < 0)
				prec = 1;
			if ((flags & SHARP) && *f == 'o' && prec <= len)
				prec = len + 1;
			pad = 0;
			if (width) {
				pad = width - (flags & SIGN ? 1 : 0) - (len < prec ? prec : len);
				if (!(flags & FLUSHLEFT)) {
					while (--pad >= 0)
						outc(' ');
				}
			}
			if (flags & SIGN)
				outc('-');
			prec -= len;
			while (--prec >= 0)
				outc('0');
			while (*p)
				outc(*p++);
			while (--pad >= 0)
				outc(' ');
			break;
		case 's':
			p = va_arg(ap, char *);
			pad = 0;
			if (width) {
				len = strlen(p);
				if (prec >= 0 && len > prec)
					len = prec;
				pad = width - len;
				if (!(flags & FLUSHLEFT)) {
					while (--pad >= 0)
						outc(' ');
				}
			}
			prec++;
			while (--prec != 0 && *p)
				outc(*p++);
			while (--pad >= 0)
				outc(' ');
			break;
		case 'c':
			c = va_arg(ap, int);
			outc(c);
			break;
		default:
			outc(*f);
			break;
		}
		f++;
	}
}
