/* 
 * prace s rtc hodinami a CMOS
 * Changelog:
 *
 * 2006-11-16: Rozdeleni na praci s datocasem a CMOS
 */

#ifndef _RTC_H_
#define _RTC_H_

#include "globals.h"

/* Pokud je zadefinovano, jsou pristupne funkce pro praci s CMOS */
#define USE_RTC_CMOS

/* Pokud je zadefinovano, jsou pristupne funkce pro praci s datem a casem */
/* #define USE_RTC_DATETIME */

/** rtc_init
 * Inicializuje rtc 
 */
void
rtc_init(void);

#ifdef USE_RTC_CMOS

/* cmos_* jsou little endian na nibble urovni */
/* adresy jsou v nibblech (v 15bajtove cmos jsou <0 .. 29>) */
void
cmos_rdnibble(byte addr, byte *val, byte *chksum);

void
cmos_rdbyte(byte addr, byte *val, byte *chksum);

void
cmos_rdword(byte addr, word *val, byte *chksum);

void
cmos_rddword(byte addr, dword *val, byte *chksum);

bool
cmos_wrnibble(byte addr, byte val, byte *chksum);

bool
cmos_wrbyte(byte addr, byte val, byte *chksum);

bool
cmos_wrword(byte addr, word val, byte *chksum);

bool
cmos_wrdword(byte addr, dword val, byte *chksum);

#endif /* USE_RTC_CMOS */

#ifdef USE_RTC_DATETIME

/** rtc_set_datetime
 * Nastavi do RTC datum a cas
 * @param datum -- datum jako rrrrmmdd
 * @param cas -- cas jako hhmmssDD (DD je den v tydnu)
 * @return 0 -- vse ok, 1 -- chyba prace s RTC
 */
byte 
rtc_set_datetime(dword datum, dword cas);

/** rtcdate
 * Zjisti z RTC datum
 * @return 0 -- chyba, jinak datum ve tvaru rrrrmmdd
 */
long 
rtcdate(void);

/** rtctime 
 * Zjisti z RTC cas
 * @return 0 -- chyba, jinak cas ve tvaru hhmmssDD (DD je den v tydnu)
 */
long
rtctime(void);

/**
 * set_string_datetime
 * Nastavi do rtc datum a cas
 * @param dt datocas ve formatu rrrr.mm.dd hh.mm
 */
void 
set_string_datetime(const char *dt);

/**
 * get_string_datetime
 * Vraci datum a cas z RTC ve tvaru rrrr.mm.dd hh:mm
 * @param dt misto pro ulozeni vysledneho stringu
 */
void 
get_string_datetime(char *dt);

#endif /* USE_RTC_DATETIME */

#endif /* _RTC_H_ */
