#include <string.h>
#include "globals.h"
#include "rtc.h"
#include "uart.h"
#include "panel.h"

/*
 * Changelog: 
 * 2006-09-26: Vykuchano z uart.c
 * 2006-09-26: Opraveno ser_in_stop
 * 2006-09-28: Opraveno ser_ibuflen
 * 2006-10-17: Pridana moznost signalizace chyb (zatim pouze preteceni)
 * 2006-10-11: Includuju <string.h> misto "block.h"
 */

#if defined(UART0) || defined(UART1)

volatile const char *txptr[2];
volatile int txcnt[2] = {0};

volatile char *rxptr[2];
volatile int rxcnt[2] = {0}, rxmax[2] = {0};
volatile char rxrun[2] = {0};

#if (UART0_BUF > 0)
volatile byte rxbuf0[UART0_BUF];
volatile byte rxbuf0cnt;
#endif 

#if (UART1_BUF > 0)
volatile byte rxbuf1[UART1_BUF];
volatile byte rxbuf1cnt;
#endif 

#ifdef UART0

static void
signal_error(byte uart, const char *err)
{
#ifdef UART_SIGNAL_ERRORS
	p_setled(4, LED_RED);
	p_move(0, 3);
	p_puts("UART ");
	p_putc(uart + '0');
	p_putc(':');
	p_puts(err);
#endif
}

__interrupt void 
uart0rx(void)
{
	if (!(IO_USR0.byte & 0x60)) {
		byte b = IO_UIDR0.DATA8;
		/* Preposila prijata data z uart1 na uart0 */
		/*
		ser_out(1, &b, 1);
		*/
		/* Tiskne data na displej */
		/*
		if (p_x > 17)
			p_move(0, p_y+1);
		if (p_y < 4) {
			p_putc(b); 
			p_putc(num2char(b >> 4)); 
			p_putc(num2char(b & 0xf));
		}
		*/

		if (rxrun[0]) {
			*rxptr[0]++ = b;
			if (++rxcnt[0] >= rxmax[0]) {
#if (UART0_BUF == 0)
				IO_USR0.bit.RIE = 0;
#endif
				rxrun[0] = 0;
				rxmax[0] = 0;
			};
		}
#if (UART0_BUF > 0)
		else {
			if (rxbuf0cnt < UART0_BUF) {
				rxbuf0[rxbuf0cnt++] = b;
			} else {
				signal_error(0, "overflow");
				rxbuf0cnt = UART0_BUF + 1; /* indikace preteceni */
				if (IO_UIDR0.DATA8) {}; /* obsluha preruseni */
			}
		}
#endif
	} else IO_UMC0.bit.RFC = 0; /* vycistim chybove flagy */
};

__interrupt void 
uart0tx(void)
{
	if (txcnt[0]) {
		if (0 != (--txcnt[0])) {
			IO_UIDR0.DATA8 = *txptr[0]++;
		}
	} else
		IO_USR0.bit.TIE = 0;
};

#endif

#ifdef UART1

__interrupt void 
uart1rx(void)
{
	if (!(IO_SSR1.byte & 0xe0)) {
		byte b = IO_SIDR1.byte;
		/* Tiskne data na displej. Je dost mozne ze diky tisku nebude lze data
		 * prijimat dost rychle. */
		/*
		if (p_x > 17)
			p_move(0, p_y+1);
		if (p_y < 4) {
			p_putc(b); 
			p_putc(num2char(b >> 4)); 
			p_putc(num2char(b & 0xf));
		}
		*/
		if (rxrun[1]) {
			*rxptr[1]++ = b;
			if (++rxcnt[1] >= rxmax[1]) {
#if (UART1_BUF == 0)
				IO_SSR1.bit.RIE = 0;
#endif
				rxrun[1] = 0;
				rxmax[1] = 0;
			}
		}
#if (UART1_BUF > 0)
		else {
			if (rxbuf1cnt < UART1_BUF)
				rxbuf1[rxbuf1cnt++] = b;
			else {
				signal_error(1, "overflow");
				rxbuf1cnt = UART1_BUF + 1; /* indikace preteceni */
				if (IO_SIDR1.byte) {}; /* obsluha preruseni */
			}
		}
#endif
	};
	IO_SCR1.bit.REC = 0;
};

__interrupt void 
uart1tx(void)
{
	if (txcnt[1]) {
		if (0 != (--txcnt[1])) {
			while(!IO_SSR1.bit.TDRE);
			IO_SIDR1.byte = *txptr[1]++;
		}
	} else {
		IO_SSR1.bit.TIE = 0;	
		/* IO_PDR5.bit.P55=0; */
	};
};


#endif

/*
 * V tomhle je dost bordel, vsechno u ceho neni zadny komentar
 * je puvodni (elsaco) a je zamyslene pro 16MHz. komentovane veci
 * (Elsaco nema nikde komentare :-) jsou ode mne a komentare snad plati.
 */
int 
ser_init(byte kanal, long baud, char parita)
{
	if (kanal > 1)
		return 3;
	rxrun[kanal] = 0;
	txcnt[kanal] = 0;
	rxmax[kanal] = 0;
	rxcnt[kanal] = 0;
	if (kanal) { /* UART1 */

#ifdef UART1
#if (UART1_BUF == 0)
		IO_SSR1.bit.RIE = 0;
#else
		IO_SSR1.bit.RIE = 1;
		rxbuf1cnt = 0;
#endif 
		IO_SSR1.bit.TIE = 0;
		IO_SMR1.byte = 0x01; /* asynchronni */
			/* pro UART1 pouzivam externi zdroj hodin a to ppg6/7.
			 * baud rate se pak spocte jako f/16 (f je frekvence ppg).
			 * Predpokladam, ze nebezim ve fazovem zavesu !!!
			 * vyuzivam ch1 (samozrejme, vystupni pin je tam.. ) */
		IO_CDCR.bit.MD	= 0;	/* vypnu prescaler */
		IO_SMR1.bit.CS	= 7;	/* external clock */

	/* PPG */
		IO_PPGC67.word		&= ~0x8080; /* PPG enable - vypneme to. */
		
		IO_PPGC67.bit.PE00	= 0; /* PPG ch0 output pin enable */
		IO_PPGC67.bit.PE10	= 1; /* PPG ch1 output pin enable */
		IO_PPGC67.bit.PIE0	= 0; /* PPG ch0 interrupt enable */
		IO_PPGC67.bit.PIE1	= 0; /* PPG ch1 interrupt enable */
		IO_PPGC67.bit.MD	= 0; /* 0 -- independent mode, 3 -- 16bit mode */

		/* BACHA, chyba v manualu co se tyce 
		 * casovych zakladen PCM a PCS, viz corrections k manualu ! 
		 * mam podezreni ze spatne jsou predevsim ty corrections :-) */
		IO_PPG67.bit.PCM	= 0; /* casova zakladna ch0 (delicka /1) */
		IO_PPG67.bit.PCS	= 0; /* casova zakladna ch1 (delicka /1) */
		
			/* bacha, kdyz v 16tibitovem modu pisu do PRL, MUSIM vzdy
			 * zapsat nejprve do ch0 a pak do ch1. Musim provest vzdy OBA
			 * zapisy !! (strana 174 HM dole) */
		switch(baud) {
#if		defined(XTAL80000)
			case 4800:
				IO_PRL[7].word		= 0x6868; break;
			case 38400:
				IO_PRL[7].word		= 0x0C0C; break;
			default: return 3;
#elif	defined(XTAL73728)
			/* Pro 4800 zrejme chybne! */
			case 4800:
				IO_PRL[7].word		= 0xC0C0;
			case 38400:
				IO_PRL[7].word		= 0x0C0C; break;
			default: return 3;
#else
#error Neznamy krystal (makro XTAL*)
#endif
		}
		IO_PPGC67.bit.PEN1	= 1; /* PPG enable - odstartujem to. */
	/* /PPG */
					   
		IO_SCR1.byte = 0x33;
		IO_SCR1.bit.PEN = parita > 0;
		IO_SCR1.bit.P = parita & 1;;
		IO_ICR14.bit.ISE = 0;
		IO_ICR14.bit.IL = 6;
		IO_PDR5.bit.P55 = 1;
		IO_DDR5.bit.D55 = 1;
		IO_DDR4.bit.D43 = 0;
#else
		return 3;
#endif /* /UART1 */
	} else {
#ifdef UART0
/* PPG */
		IO_PPGC45.word		&= ~0x8080; /* PPG enable - vypneme to. */
		
		IO_PPGC45.bit.PE00	= 0; /* PPG ch0 output pin enable */
		IO_PPGC45.bit.PE10	= 1; /* PPG ch1 output pin enable */
		IO_PPGC45.bit.PIE0	= 0; /* PPG ch0 interrupt enable */
		IO_PPGC45.bit.PIE1	= 0; /* PPG ch1 interrupt enable */
		IO_PPGC45.bit.MD	= 0; /* 0 -- independent mode, 3 -- 16bit mode */

		/* BACHA, chyba v manualu co se tyce 
		 * casovych zakladen PCM a PCS, viz corrections k manualu ! 
		 * mam podezreni ze spatne jsou predevsim ty corrections :-) */
		IO_PPG45.bit.PCM	= 0; /* casova zakladna ch0 (delicka /1) */
		IO_PPG45.bit.PCS	= 0; /* casova zakladna ch1 (delicka /1) */
		
			/* bacha, kdyz v 16tibitovem modu pisu do PRL, MUSIM vzdy
			 * zapsat nejprve do ch0 a pak do ch1. Musim provest vzdy OBA
			 * zapisy !! (strana 174 HM dole) */
		switch(baud) {
#if		defined(XTAL80000)
			case 4800:
				IO_PRL[5].word		= 0xD0D0; break;
			case 38400:
				IO_PRL[5].word		= 0x0C0C; break;
			default: return 3;
#elif	defined(XTAL73728)
			case 4800:
				IO_PRL[5].word		= 0xC0C0;
			case 38400:
				IO_PRL[5].word		= 0x0C0C; break;
			default: return 3;
#else
#error Neznamy krystal (makro XTAL*)
#endif
		}
		
		IO_PPGC45.bit.PEN1	= 1; /* PPG enable - odstartujem to. */
	/* /PPG */
		IO_DDR6.bit.D62 = 0;
#if (UART0_BUF == 0)
		IO_USR0.bit.RIE = 0;
#else
		IO_USR0.bit.RIE = 1;
		rxbuf0cnt = 0;
#endif 
		IO_USR0.bit.TIE = 0;
		IO_UMC0.byte = 0x9;
		IO_UMC0.bit.MC = (parita > 0) << 1 | 1;
		IO_UMC0.bit.PEN = parita > 0;
		IO_UIDR0.bit.P = parita & 1;
		IO_UIDR0.bit.BCH = 0x0;
		IO_UIDR0.bit.BCH0 = 0x01; /* divide by 4 */
		IO_UIDR0.bit.RC = 0x0f; /* external clock */
#if 0
		switch (baud) {
		case 4800:
			IO_UIDR0.bit.RC = 0x09;
			break;
		case 9600:
				/* Pro 4MHz (tj. bez fazoveho zavesu) ! */
			IO_UIDR0.bit.RC = 0x01;
			/* IO_UIDR0.bit.RC = 0x07; */
			break;
		case 19200:
			IO_UIDR0.bit.RC = 0x05;
			break;
		case 38400:
			IO_UIDR0.bit.RC = 0x01;
			break;
		case 62500:
			IO_UIDR0.bit.RC = 0x0b;
			break;
		default:
			return 3;
		};
#endif
		IO_ICR13.bit.ISE = 0;
		IO_ICR13.bit.IL = 6;
#else
		return 3;
#endif /* UART0 */
	};
	return 0;
};

#if 0
int 
ser_init(byte kanal, long baud, char parita)
{
	if (kanal > 1)
		return 3;
	rxrun[kanal] = 0;
	txcnt[kanal] = 0;
	if (kanal) {
#ifdef UART1
		IO_SSR1.bit.RIE = 0;
		IO_SSR1.bit.TIE = 0;
		//IO_DDR6.bit.D63 = 0; CTS ZDE VYPNUTO !!!!!!!!!!!!!
		txcnt[1] = 0;
		rxrun[1] = 0;
		IO_SMR1.byte = 0x01;
		switch (baud) {
		case 1200:
			IO_CDCR.byte = 0x88;
			IO_SMR1.bit.CS = 3;
			break;
		case 2400:
			IO_CDCR.byte = 0x88;
			IO_SMR1.bit.CS = 2;
			break;
		case 4800:
			IO_CDCR.byte = 0x88;
			IO_SMR1.bit.CS = 1;
			break;
		case 9600:
			IO_CDCR.byte = 0x88;
			IO_SMR1.bit.CS = 0;
			break;
		case 19200:
			IO_CDCR.byte = 0x8C;
			IO_SMR1.bit.CS = 0;
			break;
		case 31250:
			IO_CDCR.byte = 0x88;
			IO_SMR1.bit.CS = 4;
			break;
		case 38400:
			IO_CDCR.byte = 0x8E;
			IO_SMR1.bit.CS = 0;
			break;
		case 62500:
			IO_CDCR.byte = 0x8C;
			IO_SMR1.bit.CS = 4;
			break;
		case 76800:
			IO_CDCR.byte = 0x8F;
			IO_SMR1.bit.CS = 0;
			break;
		default:
			return 3;
		};
		IO_SCR1.byte = 0x33;
		IO_SCR1.bit.PEN = parita > 0;
		IO_SCR1.bit.P = parita & 1;;
		IO_ICR14.bit.ISE = 0;
		IO_ICR14.bit.IL = 3;
		IO_PDR5.bit.P55 = 1;
		IO_DDR5.bit.D55 = 1;
#if (UART1_BUF > 0)
		rxbuf1cnt = 0;
		IO_SSR1.bit.RIE = 1;
#endif 
#else
		return 3;
#endif
	} else {
#ifdef UART0
		IO_DDR6.bit.D62 = 0;
		IO_USR0.bit.RIE = 0;
		IO_USR0.bit.TIE = 0;
		txcnt[0] = 0;
		rxrun[0] = 0;
		IO_UMC0.byte = 0x9;
		IO_UMC0.bit.MC = (parita > 0) << 1 | 1;
		IO_UMC0.bit.PEN = parita > 0;
		IO_UIDR0.bit.P = parita & 1;
		IO_UIDR0.bit.BCH = 0x0;
		IO_UIDR0.bit.BCH0 = 0x01;
		switch (baud) {
		case 4800:
			IO_UIDR0.bit.RC = 0x09;
			break;
		case 9600:
			IO_UIDR0.bit.RC = 0x07;
			break;
		case 19200:
			IO_UIDR0.bit.RC = 0x05;
			break;
		case 38400:
			IO_UIDR0.bit.RC = 0x01;
			break;
		case 62500:
			IO_UIDR0.bit.RC = 0x0b;
			break;
		default:
			return 3;
		};
		IO_ICR13.bit.ISE = 0;
		IO_ICR13.bit.IL = 3;
#if (UART0_BUF > 0)
		rxbuf0cnt = 0;
		IO_USR0.bit.RIE = 1;
#endif 
#else
		return 3;
#endif
	};
	return 0;
}
#endif 

int 
ser_out(byte kanal, const void *co, int kolik)
{
	if (kanal > 1 || !co || !kolik)
		return 3;
	if (txcnt[kanal])
		return 1;
	txptr[kanal] = (const char *) co;
	txcnt[kanal] = kolik;
	if (kanal) {
		IO_PDR5.bit.P55=1;
		while(!IO_SSR1.bit.TDRE);
		IO_SIDR1.byte = *txptr[1]++;
		IO_SSR1.bit.TIE = 1;
	} else {
		IO_UIDR0.DATA8 = *txptr[0]++;
		IO_USR0.bit.TIE = 1;
	};
	return 0;
};

int 
ser_in(byte kanal, void *kam, int kolik)
{
	register volatile char *p = kam;
	size_t cnt = 0;

	if (!kolik)
		return 3;
	
	__DI();
	if (kanal) {
#if (UART1_BUF > 0)
		if (rxbuf1cnt > UART1_BUF) {
			/* Preteceni */
			rxbuf1cnt = UART1_BUF;
		}	
		cnt = MIN(rxbuf1cnt, kolik);
		memcpy((void *)p, (void *)rxbuf1, cnt);
		memmove((void *)rxbuf1, (void *)(rxbuf1 + cnt), rxbuf1cnt -= cnt);
#endif
		IO_SSR1.bit.RIE = 1;
	} else {
#if (UART0_BUF > 0)
		if (rxbuf0cnt > UART0_BUF) {
			/* Preteceni */
			rxbuf0cnt = UART0_BUF;
		}	
		cnt = MIN(rxbuf0cnt, kolik);
		memcpy((void *)p, (void *)rxbuf0, cnt);
		memmove((void *)rxbuf0, (void *)(rxbuf0 + cnt), rxbuf0cnt -= cnt);
#endif
		IO_USR0.bit.RIE = 1;
	}
	rxrun[kanal] = (cnt < kolik ? 1 : 0);
	rxptr[kanal] = p + cnt;
	rxmax[kanal] = kolik;
	rxcnt[kanal] = cnt;
	__EI();
	return 0;
};

int
ser_in_stop(byte kanal)
{
	if (kanal > 1)
		return 1;
	if (kanal) {
#if UART1_BUF == 0
		IO_SSR1.bit.RIE = 0;
#endif
	} else {
#if UART0_BUF == 0
		IO_USR0.bit.RIE = 0;
#endif
	}
	rxrun[kanal] = rxmax[kanal] = rxcnt[kanal] = 0;
	return 0;
}

int 
ser_ilen(byte kanal)
{
	if (kanal > 1)
		return 3;
	return rxcnt[kanal];
};

int 
ser_olen(byte kanal)
{
	if (kanal > 1)
		return 3;
	return txcnt[kanal];
};

int 
ser_cts(byte kanal)
{
	switch (kanal) {
	case 0:
		return !IO_PDR6.bit.P62;
	case 1:
		return !IO_PDR6.bit.P63;
	};
	return 0;
};

#if (UART0_BUF > 0) || (UART1_BUF > 0)

void 
ser_flushbuf(byte kanal)
{
#if (UART0_BUF > 0)
	if (kanal == 0)
		rxbuf0cnt = 0;
#endif
#if (UART1_BUF > 0)
	if (kanal == 1)
		rxbuf1cnt = 0;
#endif
}

void
ser_purgebuf(byte kanal, uint16 time)
{
	char buf;
	uint32 timer;
	for(;;) {
		timer = get_jiffies() + time;
		ser_in(kanal, &buf, 1);
		for(;;) {
			if (!ser_istat(kanal))
				break;
			go_sleep();
			if (ser_istat(kanal) && get_jiffies() > timer) {
				ser_in_stop(kanal);
				ser_flushbuf(kanal);
				return;
			}
		}
	}
}

bool
ser_isbufempty(byte kanal)
{
	return ser_ibuflen(kanal) == 0;
}

bool
ser_isoverflow(byte kanal)
{
	return ser_ibuflen(kanal) > (kanal == 0 ? UART0_BUF : UART1_BUF);
}

int
ser_ibuflen(byte kanal)
{
	if (kanal == 0)
#if (UART0_BUF > 0)
		return rxbuf0cnt;
#else
		return 0;
#endif 

	if (kanal == 1)
#if (UART1_BUF > 0)
		return rxbuf1cnt;
#else
		return 0;
#endif 
	return 0;
}

#endif  /* UART0_BUF || UART1_BUF */

int 
ser_ostat(byte kanal)
{
	if (kanal > 1)
		return 3;
	if (txcnt[kanal])
		return 1;
	return 0;
};

int 
ser_istat(byte kanal)
{
	if (kanal > 1)
		return 3;
	if (rxrun[kanal])
		return 1;
	return 0;
};

int
ser_in_timeout(byte kanal, void *data, size_t len, uint16 timeout)
{
	register uint32 timer = get_jiffies() + timeout;

	ser_in(kanal, data, len);
	while(ser_istat(kanal)) {
		if (get_jiffies() > timer) 
			return ser_ilen(kanal);
		go_sleep();
	}
	return len;
}


#endif /* UART0 || UART1 */
